#!/usr/local/bin/bash

#set -x
# pgstatspack report
# version 0.3
# by frits hoogland

if [ -z $1 ]; then
	PGUSER=$USER
else
	PGUSER=$1
fi
if [ -z $2 ]; then
	PGDB=$USER
else
	PGDB=$2
fi

psql --user $PGUSER --dbname $PGDB --quiet --command "
select * from pgstatspack_snap; 
"

printf "Enter start snapshot id : "
read STARTSNAP
printf "Enter stop snapshot id  : "
read STOPSNAP
FILENAME=/tmp/pgstatreport_${STARTSNAP}_${STOPSNAP}.txt

# heading
printf "###########################################################################################################" | tee $FILENAME
printf "\nPGStatspack version 0.3 by frits.hoogland@interaccess.nl\n" | tee -a $FILENAME
printf "###########################################################################################################\n\n" | tee -a $FILENAME

printf "Snapshot information\n" | tee -a $FILENAME
printf "Begin snapshot : \n" | tee -a $FILENAME
psql --user $PGUSER --dbname $PGDB --quiet --command "
select b.snapid, b.ts, b.description from pgstatspack_snap b where b.snapid=$STARTSNAP;
" | tee -a $FILENAME
printf "End snapshot   :\n" | tee -a $FILENAME
psql --user $PGUSER --dbname $PGDB --quiet --command "
select e.snapid, e.ts, e.description from pgstatspack_snap e where e.snapid=$STOPSNAP;
" | tee -a $FILENAME
printf "Seconds in snapshot: " | tee -a $FILENAME
psql --user $PGUSER --dbname $PGDB --quiet --tuples-only --command "
select EXTRACT(EPOCH FROM (e.ts-b.ts)) from pgstatspack_snap b, pgstatspack_snap e where b.snapid=$STARTSNAP and e.snapid=$STOPSNAP;
" | tee -a $FILENAME

printf "\nDatabase version\n" | tee -a $FILENAME
psql --user $PGUSER --dbname $PGDB --quiet --command "
select version();
" | tee -a $FILENAME

printf "Database information\n"
psql --user $PGUSER --dbname $PGDB --quiet --command "
select current_database(), pg_size_pretty(pg_database_size(current_database())) as dbsize;
" | tee -a $FILENAME

printf "\nDatabase statistics\n" | tee -a $FILENAME
psql --user $PGUSER --dbname $PGDB --quiet --command "
SELECT
	a.datname as database,
	round(CAST ( ((b.xact_commit-a.xact_commit)/(select EXTRACT(EPOCH FROM (d.ts-c.ts)) from pgstatspack_snap c, pgstatspack_snap d where c.snapid=$STARTSNAP and d.snapid=$STOPSNAP)) AS numeric) ,2) as tps,
	round(CAST ( (100*(b.blks_hit-a.blks_hit)/((b.blks_read-a.blks_read)+(b.blks_hit-a.blks_hit+1))) AS numeric) ,2) as hitrate,
	round(CAST ( (((b.blks_read-a.blks_read)+(b.blks_hit-a.blks_hit))/(select EXTRACT(EPOCH FROM (d.ts-c.ts)) from pgstatspack_snap c, pgstatspack_snap d where c.snapid=$STARTSNAP and d.snapid=$STOPSNAP)) AS numeric) ,2) as lio_ps,
	round(CAST ( ((b.blks_read-a.blks_read)/(select EXTRACT(EPOCH FROM (d.ts-c.ts)) from pgstatspack_snap c, pgstatspack_snap d where c.snapid=$STARTSNAP and d.snapid=$STOPSNAP)) as numeric),2) as pio_ps,
	round(CAST ( ((b.xact_rollback-a.xact_rollback)/(select EXTRACT(EPOCH FROM (d.ts-c.ts)) from pgstatspack_snap c, pgstatspack_snap d where c.snapid=$STARTSNAP and d.snapid=$STOPSNAP)) as numeric) ,2) as rollbk_ps
FROM
	pgstatspack_database a,
	pgstatspack_database b
WHERE
	a.snapid=$STARTSNAP
AND	b.snapid=$STOPSNAP
AND	a.datname=b.datname
ORDER BY
	tps desc;
" | tee -a $FILENAME

# get total tuples
TUPLES_TOTAL=$( psql --user $PGUSER --dbname $PGDB --tuples-only --quiet --command "
SELECT
	'aaa',SUM(((b.seq_tup_read-a.seq_tup_read)+(b.idx_tup_fetch-a.idx_tup_fetch))) as tup,
	SUM(((b.n_tup_ins-a.n_tup_ins)+(b.n_tup_ins-a.n_tup_ins))) as tup_ins,
	SUM(((b.n_tup_upd-a.n_tup_upd)+(b.n_tup_upd-a.n_tup_upd))) as tup_upd,
	SUM(((b.n_tup_del-a.n_tup_del)+(b.n_tup_del-a.n_tup_del))) as n_tup_del
FROM
	pgstatspack_tables a,
	pgstatspack_tables b
WHERE	a.snapid=$STARTSNAP
AND 	b.snapid=$STOPSNAP
AND	a.table_name=b.table_name
AND	((b.seq_tup_read is not null) and (b.idx_tup_fetch is not null))
AND NOT	( (((b.seq_tup_read-a.seq_tup_read)+(b.idx_tup_fetch-a.idx_tup_fetch))=0) );
" | grep aaa | awk '{ printf "%d", $3 }' )

printf "\nTables ordered by percentage of tuples scanned\n" | tee -a $FILENAME
psql --user $PGUSER --dbname $PGDB --quiet --command "
SELECT
	a.table_name as table,
	100*((b.seq_tup_read-a.seq_tup_read)+(b.idx_tup_fetch-a.idx_tup_fetch))/$TUPLES_TOTAL as rows_read_pct,
	100*(b.heap_blks_hit-a.heap_blks_hit)/((b.heap_blks_hit-a.heap_blks_hit)+(b.heap_blks_read-a.heap_blks_read+1)) as tab_hitrate,
	100*(b.idx_blks_hit-a.idx_blks_hit)/((b.idx_blks_hit-a.idx_blks_hit)+(b.idx_blks_read-a.idx_blks_read+1)) as idx_hitrate,
	(b.heap_blks_read-a.heap_blks_read) as tab_read,
	(b.heap_blks_hit-a.heap_blks_hit) as tab_hit,
	(b.idx_blks_read-a.idx_blks_read) as idx_read,
	(b.idx_blks_hit-a.idx_blks_hit) as idx_hit
FROM
	pgstatspack_tables a,
	pgstatspack_tables b
WHERE 
	a.snapid=$STARTSNAP
AND	b.snapid=$STOPSNAP
AND	a.table_name=b.table_name
AND	((b.seq_tup_read is not null) and (b.idx_tup_fetch is not null))
AND NOT	( (((b.seq_tup_read-a.seq_tup_read)+(b.idx_tup_fetch-a.idx_tup_fetch))<=0) )
ORDER BY
	rows_read_pct DESC;
" | tee -a $FILENAME

printf "\nIndexes ordered by scans\n"  | tee -a $FILENAME
psql --user $PGUSER --dbname $PGDB --quiet --command "
SELECT
	a.index_name as index,
	a.table_name as table,
	(b.idx_scan-a.idx_scan) as scans,
	(b.idx_tup_read-a.idx_tup_read) as tup_read,
	(b.idx_tup_fetch-a.idx_tup_fetch) as tup_fetch,
	(b.idx_blks_read-a.idx_blks_read) as idx_blks_read,
	(b.idx_blks_hit-a.idx_blks_hit) as idx_blks_hit
FROM
	pgstatspack_indexes a,
	pgstatspack_indexes b
WHERE
	a.snapid=$STARTSNAP
AND	b.snapid=$STOPSNAP
AND	a.index_name=b.index_name
AND	a.table_name=b.table_name
AND NOT(
	((b.idx_blks_read-a.idx_blks_read)<=0)     and ((b.idx_blks_hit-a.idx_blks_hit)<=0)  and
	((b.idx_tup_read-a.idx_tup_read)<=0) and ((b.idx_tup_fetch-a.idx_tup_fetch)<=0) )
ORDER BY
	scans DESC;
" | tee -a $FILENAME

printf "\nSequences ordered by blks_read\n" | tee -a $FILENAME
psql --user $PGUSER --dbname $PGDB --quiet --command "
SELECT 
	a.sequence_schema as schema,
	a.sequence_name as name,
	(b.seq_blks_read-a.seq_blks_read) as blks_read,
        (b.seq_blks_hit-a.seq_blks_hit) as blks_hit
FROM
	pgstatspack_sequences a,
	pgstatspack_sequences b
WHERE
	a.snapid=$STARTSNAP
AND	b.snapid=$STOPSNAP
AND	a.sequence_schema=b.sequence_schema
AND	a.sequence_name=b.sequence_name
AND NOT (
	((b.seq_blks_read-a.seq_blks_read)=0) and ((b.seq_blks_hit-a.seq_blks_hit)=0))
ORDER BY
	blks_read DESC;
" | tee -a $FILENAME

printf "\nParameters\n" | tee -a $FILENAME
psql --user $PGUSER --dbname $PGDB --quiet --command "
SELECT
	so.name as name,
	sa.setting as start_setting,
	so.setting as stop_setting,
	sa.source as source
FROM
	pgstatspack_settings so
LEFT OUTER JOIN 
	pgstatspack_settings sa ON ( so.name=sa.name )
WHERE
	sa.snapid=$STARTSNAP
AND	so.snapid=$STOPSNAP;
" | tee -a $FILENAME

printf "\nThis report is saved as $FILENAME\n\n"
